#!/bin/sh
#\
exec wish "$0" "$@"

####################################################################
# AuTags is a simple CD ripper which stores the content of a CD 
# into (lossless) Sun audio files. The tool can add some tags to
# these Sun audio files (without loosing the compatibility).
# The tool runs well on computers of the 486 class.
# 
# Please see the WhoAmI function some lines below for further 
# information.
#################################################################### 

# we load the msgcat package in a global scope  
package require msgcat

# we check if a language is set from the operation system
if {![info exist env(LANG)]} {set env(LANG) "en"}
msgcat::mclocale $env(LANG)
msgcat::mcload [file join [pwd] msgs]

namespace eval ::AuTags {
  namespace export Main
} 


####################################################################
# The selfie...
####################################################################
proc ::AuTags::WhoAmI {} {
  variable ABOUT

  set ABOUT(Name) "AuTags"
  set ABOUT(Year) 2022
  set ABOUT(Version) "0.0.5"

  set ABOUT(Desc) [msgcat::mc "AuTags is a simple CD ripper which \
stores the content of a CD into (lossless) Sun audio files. \
The tool can add some tags to these Sun audio files (without \
loosing the compatibility). \
The tool runs well on computers of the 486 class."]
  
  set ABOUT(Copyright) [msgcat::mc "This program was written by \
Falk Richter (falk.richter at yandex.com) in %s." $ABOUT(Year)]
  
  set ABOUT(License) [msgcat::mc "This software stands under the \
GNU GENERAL PUBLIC LICENSE and comes with NO warranty of any kind. \
Use at your own risk."]
}


####################################################################
# The mother of all procedures :o)
####################################################################
proc ::AuTags::Main {} {
  variable CONFIG; variable Local; 
  
  ::AuTags::WhoAmI
  
  ::AuTags::Startup

  ::AuTags::Frames 
 
  ::AuTags::Gui
 
#  ::AuTags::Menu
  
#  ::AuTags::BalloonHelp
  
  ::AuTags::SearchBackgroundTools
    
  # we start always with the cd ripper screen, we preset it here
  # for the first start
  set Local(LastSubProg) $Local(choiceframe).cdripper
  ::AuTags::Start_cdripper
 
  wm geometry . $CONFIG(Geometry)
  wm deiconify .
  update
}


#################################################################### 
# Here we load all need packages, set the defaults, read and parse 
# the config file
####################################################################
proc ::AuTags::Startup {} {
  variable Local; variable ABOUT;
  variable DEFAULTS; variable CONFIG;
  global env auto_path tcl_platform
 
  # not yet supported in Tcl 8.3 
  # catch {package require Tk}
  # we hide the window during build up
  wm withdraw .
 
  set Local(NameSpace) [namespace current]
  # we use a list here to allow paths with spaces
  set Local(StartDir) [list [pwd]] 
  
  # we check the existens of lib directory   
  set Local(LibDir) [list [file join [lindex $Local(StartDir) 0] lib]]
 
  if { [file exists [lindex $Local(LibDir) 0]] !="1" } {
    # show an error message and quit
	tk_messageBox -message [msgcat::mc "I can't find the program libraries!\n\nPlease install the program again."]\
    -title [msgcat::mc "Where are my libraries?"] -icon error -type ok
	::AuTags::Exit 1
  } else {
    lappend auto_path [lindex $Local(LibDir) 0]
  }
 
  # we load all needed packages
  ::AuTags::LoadPackage [list FR_messageBox FR_config2 FR_general \
  FR_multiFrame FR_msgWindow FR_emptyBox FR_balloonHelp FR_dirBox FR_comboBox \
  FR_exec] 
 
  # load all needed source files
  set SourceFiles [list [file join [lindex $Local(StartDir) 0] tcl gnudb.tcl]]
  eval lappend SourceFiles [list [file join [lindex $Local(StartDir) 0] tcl AuFile.tcl]]

  ::AuTags::LoadSrcFiles $SourceFiles
  
  # some settings for the toplevel windows
  set Local(Parent) .
  wm title $Local(Parent) [msgcat::mc "%s version %s from %s" \
  $ABOUT(Name) $ABOUT(Version) $ABOUT(Year)]
  wm protocol $Local(Parent) WM_DELETE_WINDOW ::AuTags::Exit
  
  # the default settings
  set DEFAULTS(BalloonHelp) "1"
  set DEFAULTS(Geometry) "+20+20"
  set DEFAULTS(SourceDir) $Local(StartDir)
  set DEFAULTS(ReplaceUmlaut) "1"
  set DEFAULTS(Proxy) "0"
  set DEFAULTS(ProxyHost) "dummy.org"
  set DEFAULTS(ProxyPort) "8080"
  set DEFAULTS(Domain) "my.host.com"
  set DEFAULTS(Device) "/dev/cd0a"
  set DEFAULTS(LiveRecording) "0"
  set DEFAULTS(OutputDir) $Local(StartDir)
  set DEFAULTS(Mp3Dir) $Local(StartDir) 
  set DEFAULTS(OutputFormat) "SunAudio"
  set DEFAULTS(DeleteInputFiles) "0"
  set DEFAULTS(BitRate) "128"
  set DEFAULTS(Genre) "Pop"
  set DEFAULTS(DeleteGnuDbFile) 0
  set DEFAULTS(GnuDbAddress) "http://gnudb.gnudb.org/~cddb/cddb.cgi"
  set DEFAULTS(RenamerPart1) [msgcat::mc "Titlenumber"]
  set DEFAULTS(RenamerPart2) [msgcat::mc "Artist"]
  set DEFAULTS(RenamerPart3) [msgcat::mc "Title"]
  set DEFAULTS(RenamerPart4) [msgcat::mc "Album"]
  set DEFAULTS(RenamerPart5) [msgcat::mc "Genre"]
  set DEFAULTS(RenamerPart6) [msgcat::mc "Year"]
  set DEFAULTS(RenamerReplaceSpaces) 1
  set DEFAULTS(RenamerSeparator) "::"
  set DEFAULTS(DiscIdFile) "DiscId.txt"
  set DEFAULTS(LeadingZeroAtTrackNumber) 1

  # now we set some os specific defaults 
  switch -glob -- $tcl_platform(os) {
    "NetBSD"  {set Local(OS) "NetBSD"}
    "OpenBSD" {set Local(OS) "OpenBSD"}
    "Linux"   {set Local(OS) "Linux"}
    default   {
	  # show an error message and quit
	  tk_messageBox -message [msgcat::mc "Your operation system is not yet supported."]\
	  -title [msgcat::mc "Wrong OS?"] -icon error -type ok
	  destroy .
	  exit
    }
  }

  # there are two versions of aucat, it depends on the OpenBSD version
  if {$Local(OS) == "OpenBSD"} {
    # we ask for the version
    if { [catch {exec uname -r} CatchOutput] == 0 } {
      if { [expr {double($CatchOutput)}] < 4.3 } {
		# we make our choice more precise
        set Local(OS) "OpenBSD_Old"
      }
    }
  }	

  # copy the OS type also into the AuFile-program part
  ::AuFile::SetOsType $Local(OS)

  # presetting of some namespace variables
  set Local(cdripperframe) ""
  set Local(taggerframe) ""
  set Local(playerframe) ""
  set Local(renamerframe) ""
  set Local(converterframe) ""
  
  # the standard comment in tagged files
  set Local(ConverterComment) "Created with AuTags" 
   
  # we read and parse the configuration file
  set ConfigFileDir [list [file join $env(HOME) .${ABOUT(Name)}]]
 
  if {![file exists [lindex $ConfigFileDir 0]]} {
    # create a new directory for the configuration settings
	catch {file mkdir [lindex $ConfigFileDir 0]}
  }
 
  set ConfigFile [list [file join $ConfigFileDir \
  .${ABOUT(Name)}_conf.tcl]]
 
  ::FR_config2::ReadConfig $ConfigFile $Local(NameSpace)

  # we check the config directories and set to the default values 
  # if the directory doesn't exist
  foreach Arg {SourceDir OutputDir Mp3Dir} {
	# if the directory doesn't exist...
    if { ![file isdirectory $CONFIG($Arg)] } {
      # ...we replace it with a default value
	  set CONFIG($Arg) $DEFAULTS($Arg)
    } 
  } 
}


####################################################################
# we save the configuration and finish the program
####################################################################
proc ::AuTags::Exit { {ReturnVal 0} } {
  variable CONFIG; variable Local
 
  ::FR_config2::SaveConfig $Local(NameSpace)
 
  catch {puts [msgcat::mc "Goodby!"]}
  destroy .
  exit $ReturnVal
}


####################################################################
# the package loader
# we load the needed package only once
####################################################################
proc ::AuTags::LoadPackage {Packages} {
  variable Local

  foreach Arg $Packages {
	# check if the package was already loaded
    if {![info exists Local($Arg)]} {
      # load and check for errors
	  if { [catch {package require $Arg} output] != "0" } {
        tk_messageBox -message [msgcat::mc "I can't load this part from the program library:\n\n%s\n\nPlease install the program again." $Arg]\
		 -title [msgcat::mc "Parts of the program couldn't be loaded"] -icon error -type ok
	  puts $output
        destroy .
        exit
      } else {
		# set the flag: the package is loaded
        set Local($Arg) 1
      }
    }
  }
}


####################################################################
# Source file loader
# we load the needed file only once
####################################################################
proc ::AuTags::LoadSrcFiles {SrcFiles} {
  variable Local

  foreach SrcFile $SrcFiles { 
	# we remove the directory path and the extension
	set FileName [file rootname [file tail [lindex $SrcFile 0]]] 
	# check if the file was already loaded
    if {![info exists Local($FileName)]} {
      # check if the file exists
	  if { [file exists [lindex $SrcFile 0]] != "1" } {
	    tk_messageBox -message [msgcat::mc "I can't find the program part: %s!\n\nPlease install the program again." [lindex $SrcFile 0]]\
	    -title [msgcat::mc "Where are my files?"] -icon error -type ok 
	    ::AuTags::Exit 1
      } else {
        # file exists, we load it
		source [lindex $SrcFile 0]
	    # set the flag: the file is loaded
        set Local($FileName) 1 
      }
	}
  }
}


####################################################################
# loads an external proc (and read the file if necessary)
####################################################################
proc ::AuTags::LoadProc {Procedure File} {
  variable Local
 
  set Procedure [string trimleft $Procedure [namespace current]]
  set Procedure [string trimleft $Procedure ::]
  
  # check if the procedure wasn't loaded yet
  if {[info procs $Procedure] == ""} {
    set SourceFile [list [file join [lindex $Local(StartDir) 0] tcl $File]]
  
	if { [file exists [lindex $SourceFile 0]] != "1" } {
	  # show an error message and quit
	  tk_messageBox -message [msgcat::mc "I can't find this file:\n%s\n\nPlease install the program again." [lindex $SourceFile 0]]\
      -title [msgcat::mc "Some parts are missing"] -icon error -type ok
      destroy .
      exit
    } else {	
	  source [lindex $SourceFile 0]
    }
  }
}


####################################################################
# We create the frames and set a variable for each frame - 
# so we can change the order of the frames later
####################################################################
proc ::AuTags::Frames {} {
  variable Local
 
  frame .frame1 -relief flat
  frame .frame2 -relief flat
  ::FR_multiFrame::Create frame3
  frame .frame4 -relief flat

  pack  .frame1 -side top -fill x -padx 5
  pack  .frame2 -side top -fill x -padx 5 -pady 5
  pack  .frame3 -side top -expand yes -fill both -padx 5 -pady 5
  pack  .frame4 -side top -fill x  -padx 5 -pady 5

  set Local(menuframe) "frame1"
  set Local(choiceframe) "frame2"
  set Local(mainframe) "frame3"
  set Local(msgwindow) "frame4"
}


#---This create the menu
proc ::AuTags::Menu {} \
{
  variable Local
   
  menubutton .$Local(menuframe).file -text [msgcat::mc "File"] -menu .$Local(menuframe).file.choice -underline 0
  menubutton .$Local(menuframe).pref -text [msgcat::mc "Configuration"] -menu .$Local(menuframe).pref.choice -underline 0
  menubutton .$Local(menuframe).about -text [msgcat::mc "About"] -menu .$Local(menuframe).about.choice -underline 0

  menu .$Local(menuframe).file.choice -tearoff 0
  .$Local(menuframe).file.choice add command -label [msgcat::mc "Exit"] -command {::AuTags::Exit} -accelerator "Ctr+q"
 
  
  menu .$Local(menuframe).pref.choice -tearoff 0
  .$Local(menuframe).pref.choice add command -label [msgcat::mc "Options"] -command {::AuTags::LoadProc ::AuTags::StartConfig "config.tcl"; ::AuTags::StartConfig} -accelerator "Ctr+o"
  
  menu .$Local(menuframe).about.choice -tearoff 0
  .$Local(menuframe).about.choice add command -label [msgcat::mc "About"] -accelerator "Alt+a" -command [list ::FR_messageBox::Create -type About -nspace $Local(NameSpace)]
 
  pack .$Local(menuframe).file -side left
  pack .$Local(menuframe).pref -side left
  pack .$Local(menuframe).about -side right
 
  bind . <Control-KeyPress-q> {::tk_msg::Exit}
  bind .$Local(menuframe).file.choice <Control-KeyPress-q> [list ::FR_general::MenuBinding .$Local(menuframe).file.choice 0]
  bind .$Local(menuframe).pref.choice <Control-KeyPress-o> [list ::FR_general::MenuBinding .$Local(menuframe).pref.choice 0]
  bind .$Local(menuframe).about.choice <Alt-KeyPress-a> [list ::FR_general::MenuBinding .$Local(menuframe).about.choice 0]
}


####################################################################
# create all widgets
####################################################################
proc ::AuTags::Gui {} {
  variable Local; variable CONFIG 

  # the message window
  ::FR_msgWindow::Create -window .$Local(msgwindow)

  # the multiframe buttons
  
  # the cdripper button
  button .$Local(choiceframe).cdripper -text [msgcat::mc "1. Rip a CD"] -command {::AuTags::Start_cdripper}

  button .$Local(choiceframe).tagger -text [msgcat::mc "2. Add tags"] -command {::AuTags::Start_tagger}

  button .$Local(choiceframe).player -text [msgcat::mc "3. Play files"] \
  -command {::AuTags::Start_player}
  
  button .$Local(choiceframe).renamer -text [msgcat::mc "4. Adapt file names"] \
  -command {::AuTags::Start_renamer}

  button .$Local(choiceframe).converter -text [msgcat::mc "5. Convert files"] \
  -command {::AuTags::Start_converter}
  
  # the Exit button
  button .$Local(choiceframe).quit -text [msgcat::mc "Exit"] -command {::AuTags::Exit}
 
  # place all buttons
  foreach Arg [list $Local(choiceframe).cdripper \
  $Local(choiceframe).tagger $Local(choiceframe).player \
  $Local(choiceframe).renamer $Local(choiceframe).converter \
  $Local(choiceframe).quit] {
    pack .$Arg -side left -expand yes -fill both
  }
}


####################################################################
# the start proc for the cdripper
####################################################################
proc ::AuTags::Start_cdripper {} {
  variable Local
  
  set SubProgramName "cdripper"
    
  if {$Local(cdripperframe) == ""} {
    set Local(cdripperframe) [::FR_multiFrame::NewPage $Local(mainframe) $SubProgramName]
    # load the cdripper source file
	::AuTags::LoadSrcFiles [list [file join [lindex $Local(StartDir) 0] tcl cdripper.tcl]] 
    # and call the main function	
	::CdRipper::Main $Local(cdripperframe) $Local(OS)
    ::FR_multiFrame::Size $::AuTags::Local(mainframe) $SubProgramName
  } else {
    ::CdRipper::ClearGui
  }
    
  ::FR_multiFrame::DisplayPage $::AuTags::Local(mainframe) $SubProgramName

  # we mark the active sub-program button with a blue foreground
  .$Local(LastSubProg) configure -foreground black 
  .$Local(LastSubProg) configure -activeforeground black 
  set Local(LastSubProg) $Local(choiceframe).cdripper
  .$Local(LastSubProg) configure -foreground blue 
  .$Local(LastSubProg) configure -activeforeground blue 
}


####################################################################
# the start proc for the tagger
####################################################################
proc ::AuTags::Start_tagger {} {
  variable ABOUT; variable Local
  
  set SubProgramName "tagger"
    
  if {$Local(taggerframe) == ""} {
    set Local(taggerframe) [::FR_multiFrame::NewPage $Local(mainframe) $SubProgramName]
    # load the tagger source file
	LoadSrcFiles [list [file join [lindex $Local(StartDir) 0] tcl tagger.tcl]] 
    # and call the main function	
	::AuTagger::Main $Local(taggerframe) $ABOUT(Name) $ABOUT(Version) 
  } else {
    ::AuTagger::ClearGui
  }
    
  ::FR_multiFrame::DisplayPage $Local(mainframe) $SubProgramName
  
  # we mark the active sub-program button with a blue foreground
  .$Local(LastSubProg) configure -foreground black 
  .$Local(LastSubProg) configure -activeforeground black 
  set Local(LastSubProg) $Local(choiceframe).tagger
  .$Local(LastSubProg) configure -foreground blue 
  .$Local(LastSubProg) configure -activeforeground blue 
}


####################################################################
# the start proc for the au-file player
####################################################################
proc ::AuTags::Start_player {} {
  variable Local
  
  set SubProgramName "player"
    
  if {$Local(playerframe) == ""} {
    set Local(playerframe) [::FR_multiFrame::NewPage $Local(mainframe) $SubProgramName]
    # load the player source file
	LoadSrcFiles [list [file join [lindex $Local(StartDir) 0] tcl player.tcl]] 
    # and call the main function	
	::AuPlayer::Main $Local(playerframe) $Local(OS)
  } else {
    ::AuPlayer::ClearGui
  }
    
  ::FR_multiFrame::DisplayPage $Local(mainframe) $SubProgramName
  
  # we mark the active sub-program button with a blue foreground
  .$Local(LastSubProg) configure -foreground black 
  .$Local(LastSubProg) configure -activeforeground black 
  set Local(LastSubProg) $Local(choiceframe).player
  .$Local(LastSubProg) configure -foreground blue 
  .$Local(LastSubProg) configure -activeforeground blue 
}


####################################################################
# the start proc for the au-file renamer tool
####################################################################
proc ::AuTags::Start_renamer {} {
  variable Local
  
  set SubProgramName "renamer"
    
  if {$Local(renamerframe) == ""} {
    set Local(renamerframe) [::FR_multiFrame::NewPage $Local(mainframe) $SubProgramName]
    # load the renamer source file
	LoadSrcFiles [list [file join [lindex $Local(StartDir) 0] tcl renamer.tcl]] 
    # and call the main function	
	::ReNamer::Main $Local(renamerframe)
  } else {
    ::ReNamer::ClearGui
  }
    
  ::FR_multiFrame::DisplayPage $::AuTags::Local(mainframe) $SubProgramName

  # we mark the active sub-program button with a blue foreground
  .$Local(LastSubProg) configure -foreground black 
  .$Local(LastSubProg) configure -activeforeground black 
  set Local(LastSubProg) $Local(choiceframe).renamer
  .$Local(LastSubProg) configure -foreground blue 
  .$Local(LastSubProg) configure -activeforeground blue
}


####################################################################
# the start proc to convert au-files
####################################################################
proc ::AuTags::Start_converter {} {
  variable Local
  
  set SubProgramName "converter"
    
  if {$Local(converterframe) == ""} {
    set Local(converterframe) [::FR_multiFrame::NewPage $Local(mainframe) $SubProgramName]
    # load the converter source file
	LoadSrcFiles [list [file join [lindex $Local(StartDir) 0] tcl converter.tcl]] 
    # and call the main function	
	::AuConverter::Main $Local(converterframe)
  } else {
    ::AuConverter::ClearGui
  }
    
  ::FR_multiFrame::DisplayPage $Local(mainframe) $SubProgramName
  
  # we mark the active sub-program button with a blue foreground
  .$Local(LastSubProg) configure -foreground black 
  .$Local(LastSubProg) configure -activeforeground black 
  set Local(LastSubProg) $Local(choiceframe).converter
  .$Local(LastSubProg) configure -foreground blue 
  .$Local(LastSubProg) configure -activeforeground blue 
}


####################################################################
# We check the optional background tools
####################################################################
proc ::AuTags::SearchBackgroundTools {} {
  variable Local; variable CONFIG
  global env
  
  set BackgroundTools [list Ripper cdparanoia [msgcat::mc "You can't rip audio CD's."]]

  switch -- $Local(OS) {
	"NetBSD"  { lappend BackgroundTools Player audioplay [msgcat::mc "You can't play au-files."] }
	"OpenBSD_Old" { lappend BackgroundTools Player sox [msgcat::mc "You can't play au-files."] }
	"OpenBSD" { lappend BackgroundTools Player aucat [msgcat::mc "You can't play au-files."] }
	"Linux"   { lappend BackgroundTools Player aplay [msgcat::mc "You can't play au-files."] }
  }
  
  lappend BackgroundTools Mp3Converter lame [msgcat::mc "You can't convert au-files into mp3-files."]

  ::FR_msgWindow::Add -format Heading \
  -msg [msgcat::mc "We are looking for the needed / optional background tools:"]

  foreach {Type Tool Comment} $BackgroundTools {

    if {[auto_execok $Tool] == ""} {
      ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "We can't find: %s: %s" $Tool $Comment]
      set Local($Type) [list 0 $Tool]
    } else {
      ::FR_msgWindow::Add -format Body -msg [msgcat::mc "We successfully found: %s" $Tool]
      set Local($Type) [list 1 $Tool]
    }
  } 
 
  # we check additional the permissions of cdparanoia (if cdparanoia is available)
  if {[lindex $Local(Ripper) 0]} { 
    set Permissions [file attributes [auto_execok cdparanoia] -permissions]
   
    if {(![regexp {[0-9]+4[0-9]+[0-9]+[0-9]+} $Permissions]) && ($env(USER) != "root")} {
      ::FR_msgWindow::Add -format Info -msg [msgcat::mc "Hint: You are logged in as normal user (very good!)\
but your cdparanoia havn't set the \"user-id-bit\". This can be a reason for (access-)errors during using cdparanoia.\
In this case please log in as a superuser and run the ./SetUserId.tcl script in your AuTags installation directory."]
    }
  }
}


####################################################################
# We check the background tools in the later code also: 
# to have a error message why anything don't run
####################################################################
proc ::AuTags::CheckBackgroundTools {Type} {
  variable Local

  # check if the tool was available during startup
  if {[lindex $Local($Type) 0]} {
    # we return the OS depending tool
	return [lindex $Local($Type) 1] 
  } else {
    #::FR_msgWindow::Add -format Error \
	#-msg [msgcat::mc "I can't execute this because \"%s\" isn't installed." [lindex $Local($Type) 1]]
    # return an empty list
	return ""
  }
}


####################################################################
# Switch on/off the balloonhelp
####################################################################
proc ::AuTags::BalloonHelpStatus {} {
  variable CONFIG
  
  if {$CONFIG(BalloonHelp)} {
    ::FR_balloonHelp::Activate
  } else {
    ::FR_balloonHelp::Deactivate
  }
}


####################################################################
# Returns the sorted content of a directory for a given file type
####################################################################
proc ::AuTags::CheckDirContent {Directory Format} {
  # change into the wished directory
  cd $Directory
 
  switch $Format {
    "SunAudio"   {set Files [glob -nocomplain *.{A,a}{U,u}]} \
    default      {set Files ""}
  }
  
  if {($Files != "")} {
    set Files [lsort -dictionary $Files]
  }
    
  return $Files
}


::AuTags::Main
