#-----a simple info window-----

package provide FR_messageBox 0.2

package require FR_general

namespace eval ::FR_messageBox \
{
 namespace export Create
}


proc ::FR_messageBox::Create {args} \
{
 variable VAR
 
 # Possible arguments and its default values
 # -----------------------------------------
 # -type "Error | Warning | About | Info (default)"
 # -button "Dismiss (default) | list with two elements for the two buttons"
 # -message "Your text | "Nothing" (default)"
 # -title "title of the window | "" (default)"
 # -parent "parent window | . (default)"
 # -nspace "namespace for ABOUT-variable | :: (default)"
 
 #-----first we generate a toplevel name who doesn't exist yet
 set i 1
 while {[winfo exists .messageBox_Win$i]} {incr i}
 set Path "messageBox_Win$i"
 
 
 set VAR($Path,Message) [msgcat::mc "Nothing."]
 set VAR($Path,Type) "Info"
 set VAR($Path,Parent) .
 set VAR($Path,Title) ""
 set VAR($Path,Buttons) [list [msgcat::mc "Dismiss"]]
 set VAR($Path,NameSpace) ""
  
 set VAR($Path,Stop) 0
 set VAR($Path,Return) 0 
 
 for {set i 0} {$i < [llength $args]} {incr i 2} \
 {
   set Arg [lindex $args $i]
   switch -glob -- $Arg \
   {
     -type    {set VAR($Path,Type) [lindex $args [expr {$i+1}]]} \
     -message {set VAR($Path,Message) [lindex $args [expr {$i+1}]]} \
     -parent  {set VAR($Path,Parent) [lindex $args [expr {$i+1}]]} \
     -title   {set VAR($Path,Title) [lindex $args [expr {$i+1}]]} \
     -button  {set VAR($Path,Buttons) [lindex $args [expr {$i+1}]]} \
     -nspace  {set VAR($Path,NameSpace) [lindex $args [expr {$i+1}]]} \
     --       {break} \
     *        {puts "Unknown option: $Arg"; exit 1}
   }
 }

   
 if { $VAR($Path,Type) == "Error" } \
 {
   if {$VAR($Path,Title) == ""} {set VAR($Path,Title) [msgcat::mc "Error"]}
   
   set VAR($Path,Heading) [msgcat::mc "An error occured!"]
   set VAR($Path,HeadingColor) "red"
 }
  
 if { $VAR($Path,Type) == "Info" } \
 {
   if {$VAR($Path,Title) == ""} {set VAR($Path,Title) [msgcat::mc "Information"]}
   set VAR($Path,Heading) [msgcat::mc "Info"]
   set VAR($Path,HeadingColor) "blue"
 }
  
 if { $VAR($Path,Type) == "Warning" } \
 {
   if {$VAR($Path,Title) == ""} {set VAR($Path,Title) [msgcat::mc "Warning"]}
   set VAR($Path,Heading) [msgcat::mc "Warning"]
   set VAR($Path,HeadingColor) "red"
 }
  
 if { $VAR($Path,Type) == "About" } \
 {
   if {$VAR($Path,Title) == ""} {set VAR($Path,Title) [msgcat::mc "About"]}
   set VAR($Path,HeadingColor) "blue"
   
   eval upvar $VAR($Path,NameSpace)::ABOUT ABOUT
   
   # About is the variable for the program informations which I'm using in the new programs

   foreach Arg [list Name Version Date Year Desc Contact Thanks Header License] \
   {
     if {![info exists ABOUT($Arg)]} \
     {
       set ABOUT($Arg) ""
     }
   } 

   set VAR($Path,Heading) "$ABOUT(Name)"  
   set VAR($Path,Message) $ABOUT(Header)

   append VAR($Path,Message) "\n\n" [msgcat::mc "Version: %s from %s%s" $ABOUT(Version) $ABOUT(Date) $ABOUT(Year)]

   if {$ABOUT(Thanks) != ""} \
   {  
     append VAR($Path,Message) "\n\n" [msgcat::mc "Many thanks to: %s" $ABOUT(Thanks)]
   }

   append VAR($Path,Message) "\n\n" $ABOUT(License) 
 } 

 toplevel .$Path
 wm title .$Path $VAR($Path,Title)
 wm withdraw .$Path

 frame .$Path.frame1 -borderwidth 1 -relief groove
 frame .$Path.frame2 -borderwidth 1 -relief groove

 pack  .$Path.frame1 -side top -expand yes -fill both
 pack  .$Path.frame2 -side top -fill x


 # We set a variable for each frame - so we can change the order of the frames later easier
 set VAR($Path,Main) "$Path.frame1"
 set VAR($Path,Bottom) "$Path.frame2"
 
 ::FR_general::TextwidgetWithScrollbars .$VAR($Path,Main)

 .$VAR($Path,Main).output tag configure "heading" -foreground $VAR($Path,HeadingColor) -underline yes
 .$VAR($Path,Main).output tag configure "body" -foreground black
 
 .$VAR($Path,Main).output insert end $VAR($Path,Heading) heading
 
 .$VAR($Path,Main).output insert end "\n\n" body
 .$VAR($Path,Main).output insert end $VAR($Path,Message) body
 
 .$VAR($Path,Main).output configure -state disabled
 
  
 if {[llength $VAR($Path,Buttons)] == 1} \
 {
  button .$VAR($Path,Bottom).cancel -text [lindex $VAR($Path,Buttons) 0] -state normal -command [list ::FR_messageBox::ButtonAction $Path 0]

  pack .$VAR($Path,Bottom).cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
  bind .$Path <KeyPress-Escape> [list .$VAR($Path,Bottom).cancel invoke]
 }
 
 if {[llength $VAR($Path,Buttons)] == 2} \
 {
  set ButtonText [::FR_general::ButtonsWithSameWidth [list [lindex $VAR($Path,Buttons) 0] [lindex $VAR($Path,Buttons) 1]]]
  
  button .$VAR($Path,Bottom).cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] -command [list ::FR_messageBox::ButtonAction $Path 0]

  button .$VAR($Path,Bottom).okay -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] -command [list ::FR_messageBox::ButtonAction $Path 1]
         
  pack .$VAR($Path,Bottom).okay -side left -padx 5 -pady 5 -expand yes -fill x
  pack .$VAR($Path,Bottom).cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
  bind .$Path <KeyPress-Escape> [list .$VAR($Path,Bottom).cancel invoke]
  bind .$Path <KeyPress-Return> [list .$VAR($Path,Bottom).okay invoke]
 }
 
 ::FR_general::CenterWindow .$Path $VAR($Path,Parent)
 wm deiconify .$Path
 focus .$VAR($Path,Bottom)
 
 wm protocol .$Path WM_DELETE_WINDOW [list .$VAR($Path,Bottom).cancel invoke]
 
 vwait ::FR_messageBox::VAR($Path,Stop)
 return $VAR($Path,Return)
}


#---the reaction of the ok / cancel button
proc ::FR_messageBox::ButtonAction {Path ReturnValue} \
{
  variable VAR
   
  set VAR($Path,Return) $ReturnValue   
  focus $VAR($Path,Parent)
  destroy .$Path
  set VAR($Path,Stop) 1
}

