#
#---a simple popup window
#
# -type "Entry (default)"
# -parent "<window which starts FR_popupWindow> | . (default)"
# -title "<your window title> | "Entry-box" (default)"
# -initialinput "<preload for the entry-box> | nothing (default)"
# -description "a label above the entry-box (e.g. for a description or fill out help)"
# -button "Dismiss | okcancel (default)"



package provide FR_popupWindow 0.1

package require FR_general


namespace eval ::FR_popupWindow {
  variable VAR
  namespace export Create
}


proc ::FR_popupWindow::ButtonAction {Path RETURNVALUE} {
  variable VAR
  
  if { $RETURNVALUE } {
    # in case of items with spaces we pack all into a list element
	set VAR($Path,RETURN) [list [.$VAR($Path,main).entrybox get]]
  } else {
    set VAR($Path,RETURN) ""
  }
  
  focus $VAR($Path,PARENT)
  destroy .$Path
  set VAR($Path,STOP) 1
}


proc ::FR_popupWindow::Create {args} {
  variable VAR
 
  # first we generate a toplevel name who doesn't exist yet
  set i 1
  while {[winfo exists .popup_window$i]} {incr i}
  set Path "popup_window$i" 

  # do some pre-settings
  set VAR($Path,TYPE) "Entry"
  set VAR($Path,PARENT) .
  set VAR($Path,TITLE) ""
  set VAR($Path,BUTTONS) "okcancel"
  set VAR($Path,INI_INPUT) ""
  set VAR($Path,DESCRIPTION) ""
 
  set VAR($Path,STOP) 0
  set VAR($Path,RETURN) ""
 
  # now we parse the arguments
  for {set i 0} {$i < [llength $args]} {incr i 2} {
    set arg [lindex $args $i]
    switch -glob -- $arg {
      -type         {set VAR($Path,TYPE) [lindex $args [expr {$i+1}]]} \
      -parent       {set VAR($Path,PARENT) [lindex $args [expr {$i+1}]]} \
      -title        {set VAR($Path,TITLE) [lindex $args [expr {$i+1}]]} \
      -initialinput {set VAR($Path,INI_INPUT) [lindex $args [expr {$i+1}]]} \
      -description  {set VAR($Path,DESCRIPTION) [lindex $args [expr {$i+1}]]} \
      -button       {set VAR($Path,BUTTONS) [lindex $args [expr {$i+1}]]} \
      --            {break} \
      *             {puts "Unknown option: $arg"; exit 1}
    }
  } 
   
  if { $VAR($Path,TYPE) == "Entry" } {
    if {$VAR($Path,TITLE) == ""} {set VAR($Path,TITLE) [msgcat::mc "Entry-box"]}
  }

  toplevel .$Path
  wm title .$Path $VAR($Path,TITLE)
  wm withdraw .$Path

  frame .$Path.frame1 -relief flat
  frame .$Path.frame2 -relief flat

  pack  .$Path.frame1 -side top -expand yes -fill both -padx 5 -pady 5
  pack  .$Path.frame2 -side top -fill x

  # We set a variable for each frame - so we can change the order of the frames later easier 
  set VAR($Path,main) "$Path.frame1"
  set VAR($Path,bottom) "$Path.frame2"
 
  label .$VAR($Path,main).description -justify left -text $VAR($Path,DESCRIPTION)
 
  entry .$VAR($Path,main).entrybox -width 35 -background white -xscrollcommand ".$VAR($Path,main).xsbar set"
  scrollbar .$VAR($Path,main).xsbar -orient horizontal -command ".$VAR($Path,main).entrybox xview"

  grid .$VAR($Path,main).description -row 0 -sticky ws -padx 5 -pady 5
  grid .$VAR($Path,main).entrybox -row 1 -sticky ew -padx 5
  grid .$VAR($Path,main).xsbar -row 2 -sticky ew -padx 5
  grid columnconfigure .$VAR($Path,main) 0 -weight 1
  grid rowconfigure .$VAR($Path,main) 0 -weight 1

  
  if { $VAR($Path,INI_INPUT) != "" } {
    .$VAR($Path,main).entrybox insert end $VAR($Path,INI_INPUT)
    .$VAR($Path,main).entrybox selection range 0 end
  }
 
  if { $VAR($Path,BUTTONS) == "Dismiss" } {
    button .$VAR($Path,bottom).cancel -text [msgcat::mc "Dismiss"] -state normal \
	-command [list ::FR_popupWindow::ButtonAction $Path 0]

    pack .$VAR($Path,bottom).cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
    bind .$Path <KeyPress-Escape> [list .$VAR($Path,bottom).cancel invoke]
  }
 
  if { $VAR($Path,BUTTONS) == "okcancel" } {
    set BUTTONTEXT [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
  
    button .$VAR($Path,bottom).cancel -text [lindex $BUTTONTEXT 1] -state normal \
	-width [lindex $BUTTONTEXT 2] -command [list ::FR_popupWindow::ButtonAction $Path 0]
   
    button .$VAR($Path,bottom).okay -text [lindex $BUTTONTEXT 0] -state normal \
    -width [lindex $BUTTONTEXT 2] -command [list ::FR_popupWindow::ButtonAction $Path 1]
   
    pack .$VAR($Path,bottom).okay -side left -padx 5 -pady 5 -expand yes -fill x
    pack .$VAR($Path,bottom).cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
    bind .$Path <KeyPress-Escape> [list .$VAR($Path,bottom).cancel invoke]
    bind .$Path <KeyPress-Return> [list .$VAR($Path,bottom).okay invoke]
  }
 
  ::FR_general::CenterWindow .$Path $VAR($Path,PARENT)
  wm deiconify .$Path
  wm protocol .$Path WM_DELETE_WINDOW [list .$VAR($Path,bottom).cancel invoke]
 
  focus .$VAR($Path,main).entrybox
 
  vwait ::FR_popupWindow::VAR($Path,STOP)
  return $VAR($Path,RETURN)
}

