#*****************************************************************************
# Script renames (extended) Sun audio files 
#
# Author: Falk Richter, falk.richter@yandex.com, 2021 
# $Id: renamer.tcl,v 1.2 2021/07/07 21:36:02 falk Exp falk $
#***************************************************************************** 

namespace eval ::ReNamer {
  variable Local; 
  variable Config;
  # we need only this functions from the outside
  namespace export Main ClearGui

  upvar ::AuTags::CONFIG ::ReNamer::Config
}


####################################################################
# The main proc for the au-player
#################################################################### 
proc ::ReNamer::Main { Frame } {
  variable Local
  
  # we copy the in AuTags generated frame for the renamer tool
  set Local(renamerframe) $Frame 
  
  set Local(AllFiles) "" 
  
  ::ReNamer::Gui
}


####################################################################
# after a switch to this program part we make some presettings
####################################################################
proc ::ReNamer::ClearGui {} {
  variable Local
  
  .$Local(renamerframe).start configure -state disabled
}


####################################################################
# we create and place all gui elements
####################################################################
proc ::ReNamer::Gui {} {
  variable Local; variable Config
    
  label .$Local(renamerframe).outputdir_comment -text [msgcat::mc "Choose the directory with your au-files:"] -pady 10
  button .$Local(renamerframe).outputdir -text [msgcat::mc "Open"] -command {::ReNamer::OutputDir}
 
  label .$Local(renamerframe).options_comment -text [msgcat::mc "Choose the parts of the new file name:"] -pady 10

  # we generate a short form
  set choiceFrame $Local(renamerframe).parts
  frame .$choiceFrame

  set OptionsList [list [msgcat::mc "Nothing"] [msgcat::mc "Titlenumber"] [msgcat::mc "Artist"] [msgcat::mc "Title"] \
  [msgcat::mc "Album"] [msgcat::mc "Year"] [msgcat::mc "Genre"] "CD1" "CD2"]

  for {set i 1} {$i < 7} {incr i} {
    label .$choiceFrame.renamer_comment$i -text [msgcat::mc "Part: %i" $i]

    ::FR_comboBox::cb .$choiceFrame.renamer_part$i -variable ::ReNamer::Config(RenamerPart$i)
    ::FR_comboBox::cb .$choiceFrame.renamer_part$i -listbox_content $OptionsList -entry_width auto -default_value $Config(RenamerPart$i)
  }

  checkbutton .$Local(renamerframe).spacereplacement -text [msgcat::mc "Replace spaces in the content by underlines"] \
  -variable ::ReNamer::Config(RenamerReplaceSpaces) -pady 10

  label .$Local(renamerframe).separator_comment -pady 10 -text [msgcat::mc "Choose the kind of separator between the parts:"]

  set SeparatorList [list "::" "__" [msgcat::mc "2x space"] ":" "_" [msgcat::mc "1x space"]]

  ::FR_comboBox::cb .$Local(renamerframe).separator -variable ::ReNamer::Config(RenamerSeparator)
  ::FR_comboBox::cb .$Local(renamerframe).separator -listbox_content $SeparatorList -entry_width auto -default_value $Config(RenamerSeparator)
 
  button .$Local(renamerframe).start -text [msgcat::mc "Start"] -state disabled -command {::ReNamer::Start}
  
  # the placement 
  grid .$Local(renamerframe).outputdir_comment -row 0 -column 0 -sticky w 
  grid .$Local(renamerframe).outputdir -row 0 -column 2 -sticky ew
  
  ::FR_general::Line $Local(renamerframe) 1 3 

  grid .$Local(renamerframe).options_comment -row 2 -column 0 -sticky w
  grid .$choiceFrame -row 3 -column 0 -columnspan 3 -sticky ew
 
  # the placement in the choiceFrame 
  grid .$choiceFrame.renamer_comment1 -row 0 -column 0 -sticky w
  grid .$choiceFrame.renamer_comment2 -row 0 -column 1 -sticky w -padx 5 
  grid .$choiceFrame.renamer_comment3 -row 0 -column 2 -sticky w -padx 5
  grid .$choiceFrame.renamer_comment4 -row 0 -column 3 -sticky w -padx 5
  grid .$choiceFrame.renamer_comment5 -row 0 -column 4 -sticky w -padx 5
  grid .$choiceFrame.renamer_comment6 -row 0 -column 5 -sticky w

  grid .$choiceFrame.renamer_part1 -row 1 -column 0 -sticky w -pady 5
  grid .$choiceFrame.renamer_part2 -row 1 -column 1 -sticky w -pady 5 -padx 5 
  grid .$choiceFrame.renamer_part3 -row 1 -column 2 -sticky w -pady 5 -padx 5
  grid .$choiceFrame.renamer_part4 -row 1 -column 3 -sticky w -pady 5 -padx 5
  grid .$choiceFrame.renamer_part5 -row 1 -column 4 -sticky w -pady 5 -padx 5
  grid .$choiceFrame.renamer_part6 -row 1 -column 5 -sticky w -pady 5
 
  # we come back to the renamerframe
  grid .$Local(renamerframe).spacereplacement -row 4 -column 0 -sticky w
  grid .$Local(renamerframe).separator_comment -row 5 -column 0 -sticky w 
  grid .$Local(renamerframe).separator -row 5 -column 2 -sticky ew

  eval ::FR_general::PadFrame $Local(renamerframe) 6 3

  grid .$Local(renamerframe).start -row 7 -column 0 -columnspan 3 -sticky ew

  grid columnconfigure .$Local(renamerframe) 1 -weight 1
  grid columnconfigure .$choiceFrame 5 -weight 1
 
  ::FR_balloonHelp::AddHelp ".$Local(renamerframe).outputdir" [msgcat::mc "This is the directory with your au-files"]
  ::FR_balloonHelp::AddHelp ".$Local(renamerframe).separator" [msgcat::mc "You can choice the separator between two parts"]
  ::FR_balloonHelp::AddHelp ".$Local(renamerframe).spacereplacement" \
  [msgcat::mc "If it's activated: all spaces in the content (not the separator spaces) are replaced by underlines"] 

  for {set i 1} {$i  < 7} {incr i} {
    ::FR_balloonHelp::AddHelp ".$choiceFrame.renamer_part$i" [msgcat::mc "This sets the part %i of the new file name" $i]
  }
   
  ::AuTags::BalloonHelpStatus
}


####################################################################
# load the output directory
####################################################################
proc ::ReNamer::OutputDir {} {
  variable Config; variable Local

  set OutputDir [::FR_dirBox::Create -initialdir $Config(OutputDir) -parent .$Local(renamerframe) -permission "writable" \
  -title [msgcat::mc "Choose the directory with your compressed files:"] -filetype [list [msgcat::mc "Sun audio files"] "*.au"]]

  if {[lindex $OutputDir 0] != ""} {
    set Config(OutputDir) $OutputDir
    
    cd [lindex $Config(OutputDir) 0]

    ::FR_msgWindow::Add -format Info -msg [msgcat::mc "The directory was set to: %s" [lindex $Config(OutputDir) 0]]
    
    ::ReNamer::DirContent
  }
}


####################################################################
# create and check the a list with the rename and separator wishes
####################################################################
proc ::ReNamer::FetchSettings {} {
  variable Config; variable Local

  set PartsList ""
 
  for {set i 1} {$i < 7} {incr i} {
    lappend PartsList $Config(RenamerPart$i)
  }

  set PartsListSorted [lsort -unique $PartsList]

  if {([llength $PartsListSorted] == 1) && ([lindex $PartsListSorted 0] == [msgcat::mc "Nothing"])} {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "No renaming possible because all fields are set to \"Nothing\""]
  
    set Local(RenamerPartsList)  ""
 
    return 1
  } else {
    set Local(RenamerPartsList) $PartsList
  }
 
  eval switch {$Config(RenamerSeparator)} {
    [msgcat::mc "1x space"] {set Local(RenamerSeparator) " "} \
    [msgcat::mc "2x space"] {set Local(RenamerSeparator) "  "} \
    default                 {set Local(RenamerSeparator) $Config(RenamerSeparator)} \
  }
 
  return 0
}


####################################################################
# the start button
####################################################################
proc ::ReNamer::Start {} {
  variable Local
  
  if {[::ReNamer::FetchSettings]} { return } 
  
  if { [::ReNamer::ReadTags] } {
    ::FR_messageBox::Create -title [msgcat::mc "End of job"] -type Error \
	-message [msgcat::mc "An error occured during renaming. Please check the message window for further information."] 
  } else {
    ::FR_messageBox::Create -message [msgcat::mc "All files have been successful renamed."] -title [msgcat::mc "End of job"] -type Info
  }
  
  .$Local(renamerframe).start configure -state disabled
}


####################################################################
# activates the start button if the directory contains SunAudio 
# files
####################################################################
proc ::ReNamer::DirContent {} {
  variable Local; variable Config
  
  set Local(AllFiles) [::AuTags::CheckDirContent [lindex $Config(OutputDir) 0] "SunAudio"]
 
  if {$Local(AllFiles) == ""} {
    .$Local(renamerframe).start configure -state disabled
    ::FR_msgWindow::Add -format Info -msg [msgcat::mc "No au-files found"]
  } else {
    .$Local(renamerframe).start configure -state normal
    ::FR_msgWindow::Add -format Info -msg [msgcat::mc "%i au-file(s) found" [llength $Local(AllFiles)]]
  }
}


####################################################################
# read out the tags of the files
####################################################################
proc ::ReNamer::ReadTags {} {
  variable Local; variable Config
  
  set Index 1
 
  foreach File $Local(AllFiles) {
    ::FR_msgWindow::Add -format Heading -msg [msgcat::mc "Now, we rename the file: %s" $File]

    # now we read the tags from the au-files
	set TagList [::AuFile::ReadTags [file join [lindex $Config(OutputDir) 0] $File]]
     
    # we check the success
	if { [lindex $TagList 0] != 0 } {
      # oops an error occurred - we ask for the reason
	  set ErrorList [::AuFile::GetErrorMsg]
    
	  ::FR_msgWindow::Add -format Error -msg [msgcat::mc "An error occurred during reading of file: %s" $File]
      ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "The reason: %s" [lindex $ErrorList 0]] 
      ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "Details: %s" [lindex $ErrorList 1]]

	  # we stop here
	  return 1
	} 

    # now we try to rename the current file
	if { [::ReNamer::RenameFiles [file join [lindex $Config(OutputDir) 0] $File] $TagList $Index] } {
	  # an error occurred, we stop here
	  return 1
	}
    
    # we prepare the next loop 
    incr Index
  }
  
  return 0
}


####################################################################
# renaming with the handed over file tags
####################################################################
proc ::ReNamer::RenameFiles {File TagList Index} {
  variable Local; variable Config
 
  # the first element in the tags list is the return value of the tags read function,
  # we check only the rest
  for {set i 1} {$i < [llength $TagList]} {incr i} {
	# check if elements in the tags list are empty
    if { [lindex $TagList $i] == "" } {
      # empty elements are replaced by "unknown"
	  lreplace TagList $i $i "unknown"
    }
  }
 
  # the structure of the TagsList is: ReturnValue Title Artist Album Comment Year Track Genre (in form of text)
  set FileNameParts ""

  for {set i 0} {$i < [llength $Local(RenamerPartsList)]} {incr i} {
    eval switch [lindex $Local(RenamerPartsList) $i] {
      [msgcat::mc "Title"]       {lappend FileNameParts [lindex $TagList 1]} \
      [msgcat::mc "Artist"]      {lappend FileNameParts [lindex $TagList 2]} \
      [msgcat::mc "Album"]       {lappend FileNameParts [lindex $TagList 3]} \
      [msgcat::mc "Year"]        {lappend FileNameParts [lindex $TagList 5]} \
      [msgcat::mc "Titlenumber"] {lappend FileNameParts [::ReNamer::EditTrackNumber [lindex $TagList 6]]} \
      [msgcat::mc "Genre"]       {lappend FileNameParts [lindex $TagList 7]} \
	  "CD1"                      {lappend FileNameParts "CD1"} \
	  "CD2"                      {lappend FileNameParts "CD2"} \
    }
  } 
    
  # now we compose the file name
  for {set i 0} {$i < [llength $FileNameParts]} {incr i} {
	# fetch the part of the name
    set Tmp [lindex $FileNameParts $i]
  
    # replace the spaces if wished
	if {$Config(RenamerReplaceSpaces)} {
      regsub -all -- {\s} $Tmp {_} Tmp
    }
  
    if {$i == 0} {
      # this is the start of the file name, no separator is needed
      set FileName $Tmp
    } else {
      # normally we need a separator
	  append FileName $Local(RenamerSeparator) $Tmp
    }
  }

  # add the file extension
  append FileName [file extension $File]
  
  # add the full path
  set FileName [file join [lindex $Config(OutputDir) 0] $FileName]
  
  if {[file exists $FileName]} {
    ::FR_msgWindow::Add -format BodyRed \
	-msg [msgcat::mc "I can't rename the file: %s to %s because the new filename already exists." [file tail $File] [file tail $FileName]]
    
    return 1
  }
 
  # finally we rename the file
  file rename $File $FileName

  ::FR_msgWindow::Add -format Body -msg [msgcat::mc "Finished renaming to: %s" [file tail $FileName]]
 
  return 0
}


####################################################################
# if wished: add a zero in front of the single digit number
####################################################################
proc ::ReNamer::EditTrackNumber {TrackNumber} {
  variable Config

  # check if the leading zero is wished
  if { $Config(LeadingZeroAtTrackNumber) } {
	# convert the track number to integer
	set TrackNumber [expr {int($TrackNumber)}]
	# check if the track number is a single digit number
	if { $TrackNumber < 10 } {
	  return "0$TrackNumber"
    }
  }
  # in all other cases (track number is not a single digit number
  # or the leading zero is not wished): we return the track number "as is"
  return $TrackNumber 
}

