#*****************************************************************************
# Script tags (extended) Sun audio files 
#
# Author: Falk Richter, falk.richter@yandex.com, 2021 
# $Id: tagger.tcl,v 1.3 2021/12/08 20:24:33 falk Exp falk $
#***************************************************************************** 

# needed for gnu database access
package require http

namespace eval ::AuTagger {
  variable Local; 
  variable Config;
  # we need only this functions from the outside
  namespace export Main ClearGui

  upvar ::AuTags::CONFIG ::AuTagger::Config
}


####################################################################
# The main proc for the au-tagger
#################################################################### 
proc ::AuTagger::Main { Frame ProgName ProgVersion } {
  variable Local
  
  # we copy the in AuTags generated frame for the tagger
  set Local(taggerframe) $Frame

  # we copy the name of the program
  set Local(ProgName) $ProgName
  
  # we copy the version of the program
  set Local(ProgVersion) $ProgVersion
  
  # the count of the (later generated) title fields
  set Local(TitleFields) 0
  
  # the list with all files of the chosen directory
  set Local(AllFiles) ""
  
  # Create the gui
  ::AuTagger::Gui
}


####################################################################
# after a switch to this program part we make some presettings
####################################################################
proc ::AuTagger::ClearGui {} {
  variable Local
   
  # clear the fields which are common for all titles
  .$Local(CommonTagsFrame).artist delete 0 end
  .$Local(CommonTagsFrame).album delete 0 end
  .$Local(CommonTagsFrame).year delete 0 end
  
  # insert the current year as preset
  .$Local(CommonTagsFrame).year insert end [clock format [clock seconds] -format %Y]
  
  # we destroy all existing widgets in the scroll frame
  if { $Local(TitleFields) != 0 } {
    # there exists title fields
	foreach Widget [winfo children .$Local(TitleFrame)] {
      catch {destroy $Widget}
    }
    
    set Local(TitleFields) 0
  }
  
  ::FR_general::MakeDisable .$Local(taggerframe).start .$Local(taggerframe).gnudb
}


####################################################################
# we create and place all gui elements
####################################################################
proc ::AuTagger::Gui {} {
  variable Local; variable Config
  
  # the widgets 
  label .$Local(taggerframe).outputdir_comment  -pady 10 \
  -text [msgcat::mc "Choose the directory with your au-files:"]
  button .$Local(taggerframe).outputdir -text [msgcat::mc "Open"] -command {::AuTagger::OutputDir}
    
  label .$Local(taggerframe).gnudb_comment -pady 10 -text [msgcat::mc "Get the information from the GnuDB internet database:"]
  button .$Local(taggerframe).gnudb -text [msgcat::mc "Send request"] -command {::AuTagger::ReadGnuDb}

  set Local(CommonTagsFrame) "$Local(taggerframe).commontags"
  frame .$Local(CommonTagsFrame) -relief flat
  
  set Local(TitleTagsFrame) "$Local(taggerframe).titletags"
  frame .$Local(TitleTagsFrame) -relief flat
  
  button .$Local(taggerframe).start -text [msgcat::mc "Write tags"] -foreground blue -state disabled -command {::AuTagger::StartButton}
  
  # the placement 
  grid .$Local(taggerframe).outputdir_comment -row 0 -column 0 -sticky w 
  grid .$Local(taggerframe).outputdir -row 0 -column 2 -sticky ew
  
  ::FR_general::Line $Local(taggerframe) 1 3
  
  grid .$Local(taggerframe).gnudb_comment -row 2 -column 0 -sticky w 
  grid .$Local(taggerframe).gnudb -row 2 -column 2 -sticky ew
  
  ::FR_general::Line $Local(taggerframe) 3 3
  
  grid .$Local(CommonTagsFrame) -row 4 -column 0 -columnspan 3 -sticky nsew
  
  grid .$Local(TitleTagsFrame) -row 5 -column 0 -columnspan 3 -sticky nsew -pady 5
  
  grid .$Local(taggerframe).start -row 6 -column 0 -columnspan 3 -sticky ew
  
  grid columnconfigure .$Local(taggerframe) 1 -weight 1
  grid rowconfigure .$Local(taggerframe) 5 -weight 1 
  
  # the widgets of the common tags frame
  label .$Local(CommonTagsFrame).common_comment -pady 5 -text [msgcat::mc "Tags which are used for all songs:"]
  
  label .$Local(CommonTagsFrame).artist_comment -text [msgcat::mc "Artist:"]
  entry .$Local(CommonTagsFrame).artist -state normal -width 28

  label .$Local(CommonTagsFrame).album_comment -text [msgcat::mc "Album:"]
  entry .$Local(CommonTagsFrame).album -state normal -width 28

  label .$Local(CommonTagsFrame).year_comment -text [msgcat::mc "Year:"]
  entry .$Local(CommonTagsFrame).year -state normal -width 4

  .$Local(CommonTagsFrame).year insert end [clock format [clock seconds] -format %Y]

  label .$Local(CommonTagsFrame).genre_comment -text [msgcat::mc "Genre:"]
  ::FR_comboBox::cb .$Local(CommonTagsFrame).genre -variable ::AuTagger::Config(Genre)
  ::FR_comboBox::cb .$Local(CommonTagsFrame).genre -listbox_content $::AuFile::GenreCat -entry_width auto -default_value $Config(Genre)
  
  # we put the label for the next frame here
  label .$Local(CommonTagsFrame).titletags_comment -pady 5 -text [msgcat::mc "Specific tags for each song:"]
  
  # the placement of the common tags frame
  grid .$Local(CommonTagsFrame).common_comment -row 0 -column 0 -columnspan 5 -sticky w
  grid .$Local(CommonTagsFrame).artist_comment -row 1 -column 0 -sticky w 
  grid .$Local(CommonTagsFrame).artist -row 1 -column 1 -columnspan 4 -sticky ew
  grid .$Local(CommonTagsFrame).album_comment -row 2 -column 0 -sticky w 
  grid .$Local(CommonTagsFrame).album -row 2 -column 1 -columnspan 4 -sticky ew
  grid .$Local(CommonTagsFrame).year_comment -row 3 -column 0 -sticky w 
  grid .$Local(CommonTagsFrame).year -row 3 -column 1 -sticky w
  grid .$Local(CommonTagsFrame).genre_comment -row 3 -column 3 -sticky w 
  grid .$Local(CommonTagsFrame).genre -row 3 -column 4 -sticky w
  grid .$Local(CommonTagsFrame).titletags_comment -row 4 -column 0 -columnspan 5 -sticky w
  
  grid columnconfigure .$Local(CommonTagsFrame) 1 -weight 1
 
  # the scroll window
  set Local(TitleFrame) [::FR_general::ScrollWindow $Local(TitleTagsFrame)]

  
  ::FR_balloonHelp::AddHelp ".$Local(taggerframe).outputdir" [msgcat::mc "This is the directory with your Sun audio files"]
  ::FR_balloonHelp::AddHelp ".$Local(taggerframe).gnudb" [msgcat::mc "GnuDB is the successor of the (better known) freedb database"]
  ::FR_balloonHelp::AddHelp ".$Local(CommonTagsFrame).artist" [msgcat::mc "If all tracks have the same artist - please put it in here"]
  
  ::AuTags::BalloonHelpStatus
}


####################################################################
# load the output directory
####################################################################
proc ::AuTagger::OutputDir {} {
  variable Config; variable Local

  set OutputDir [::FR_dirBox::Create -initialdir [lindex $Config(OutputDir) 0] -parent .$Local(taggerframe) \
  -title [msgcat::mc "Choose the directory with your au-files:"] -permission "writable" \
  -filetype [list [msgcat::mc "Sun audio files"] "*.au"]]
  
  # check if the answer is empty
  if {[lindex $OutputDir 0] != ""} {
	# copy the new directory (in list format)
    set Config(OutputDir) $OutputDir
    
    cd [lindex $Config(OutputDir) 0]

    ::FR_msgWindow::Add -format Info \
	-msg [msgcat::mc "The directory was set to: %s" [lindex $Config(OutputDir) 0]]
    
    ::AuTagger::TitleFieldsCreate
  }
}


####################################################################
# check if all elements of a list are equal
####################################################################
proc ::AuTagger::ElementsAllEqual {List} {
  
  # remove all double values
  set SortedList [lsort -unique $List]

  # if all elements are equal, only one list element keeps left
  if {[llength $SortedList] == 1} {
    # we return the element
	return [lindex $SortedList 0]
  } else {
    # different elements in the list
	return ""
  }
}


####################################################################
# fill the entry widgets with new contents
####################################################################
proc ::AuTagger::InsertEntryContent {Widget {Value ""}} {
  # check if the widget really exists
  if { [winfo exists $Widget] } {
    # we delete the old content
	$Widget delete 0 end
    
    if {$Value != ""} { 
	  # we fill in only real content
      $Widget insert end $Value
    }
  }
} 
 

####################################################################
# the start button
####################################################################
proc ::AuTagger::StartButton {} {
  variable Config
  
  ::AuTagger::WriteInfo
   
  # we read the au files again to control the write success
  ::AuTagger::TitleFieldsCreate
  
  if {$Config(DeleteGnuDbFile)} {
    ::AuTagger::DeleteGnuDbFile
  }

  ::FR_messageBox::Create -message [msgcat::mc "We have written and re-read all tags.\nYou can now control the correctness."] \
	-title [msgcat::mc "End of job"] -type Info
}


####################################################################
# generate one line per song and fill the entry-widgets with the 
# title/artist information
####################################################################
proc ::AuTagger::TitleFieldsCreate {} {
  variable Config; variable Local
  
  # we destroy all existing widgets in the scrollframe
  if { $Local(TitleFields) != 0 } {
    foreach Widget [winfo children .$Local(TitleFrame)] {
      catch {destroy $Widget}
    }
    # reset the count of title fields
    set Local(TitleFields) 0
  }
  
  set Local(AllFiles) [::AuTags::CheckDirContent [lindex $Config(OutputDir) 0] "SunAudio"]

  # check if we found something
  if {$Local(AllFiles) == ""} {
	# deactivate the start and the GnuDB button
    ::FR_general::MakeDisable .$Local(taggerframe).start .$Local(taggerframe).gnudb

    .$Local(CommonTagsFrame).artist delete 0 end
    .$Local(CommonTagsFrame).album delete 0 end
    .$Local(CommonTagsFrame).year delete 0 end
    
    .$Local(CommonTagsFrame).year insert end [clock format [clock seconds] -format %Y]
    
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "I didn't find any au-files. Have you chosen the right directory?"]
    # we can stop here
	return
  } 
   
  # the labels in the head of the table
  label .$Local(TitleFrame).file_comment -text [msgcat::mc "Filename"]
  label .$Local(TitleFrame).artist_comment -text [msgcat::mc "Artist"]
  label .$Local(TitleFrame).title_comment -text [msgcat::mc "Title"]

  # we place the header
  grid .$Local(TitleFrame).file_comment -row 0 -column 0 -sticky w
  grid .$Local(TitleFrame).artist_comment -row 0 -column 1 -sticky w 
  grid .$Local(TitleFrame).title_comment -row 0 -column 2 -sticky w
	
  # erase the "all equal" lists
  set AllArtists ""; set AllAlbums ""; set AllGenres ""; set AllYears ""
  # we save the read track number here
  set Local(AllTracks) ""
  # the start index
  set Index 0
    
  # now we process each file
  foreach File $Local(AllFiles) { 
    # for a good looking of the showed file name: 1. Song; 10.Song 
    #if { $Index < 10 } {
    #  set FileName "${Index}. "
    #} else {
	#  set FileName "${Index}."
    #}

    # we limit the count of characters in the file name to 20
    if { [string length $File] > 20 } {
	  # split file name into name and extension
      set RootName [file rootname $File]
      set FileExtension [file extension $File] 
      # range from 0...18 are 19 characters + "~" = 20
      # we reduce the file name length by the extension length
	  set NameLimit [expr {18 - [string length $FileExtension]}]
      # we add all togther again to the index
      set FileName [string range $RootName 0 $NameLimit] 
	  append FileName "~" $FileExtension
    } else {
	  # we add the file name to the above created index
      set FileName $File
    }
   
    # the file depending widgets
	label .$Local(TitleFrame).file$Index -text "$FileName"
    entry .$Local(TitleFrame).artist$Index -state normal -width 28 -validate focusin \
	-validatecommand "::FR_general::ScrollWindowShowWidget $::AuTagger::Local(TitleTagsFrame) %W"
      
	entry .$Local(TitleFrame).title$Index -state normal -width 28 -validate focusin \
	-validatecommand "::FR_general::ScrollWindowShowWidget $::AuTagger::Local(TitleTagsFrame) %W"

	# the first index is zero but the first row is the header, so the row is index plus one
	set Row [expr {$Index + 1}]
	grid .$Local(TitleFrame).file$Index -row $Row -column 0 -sticky w
	grid .$Local(TitleFrame).artist$Index -row $Row -column 1 -sticky ew
	grid .$Local(TitleFrame).title$Index -row $Row -column 2 -sticky ew
	 
    # now we read the tags from the au-files
	set TagList [::AuFile::ReadTags [file join [lindex $Config(OutputDir) 0] $File]]
     
    # we check the success
	if { [lindex $TagList 0] != 0 } {
      # oops an error occurred - we ask for the reason
	  set ErrorList [::AuFile::GetErrorMsg]
    
	  ::FR_msgWindow::Add -format Error -msg [msgcat::mc "An error occurred during reading of file: %s" $File]
      ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "The reason: %s" [lindex $ErrorList 0]] 
      ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "Details: %s" [lindex $ErrorList 1]]

	  # we need the count of widgets to destroy all at the next start of titleartist,
      # Index is zero based so we use Row because it is Index + 1
	  set Local(TitleFields) $Row
    
      return
    }

    # we generate lists with the information of all au-files
	# the form: success Title Artist Album Comment Year Track Genre (in form of text) 
    .$Local(TitleFrame).title$Index insert end [lindex $TagList 1]
    .$Local(TitleFrame).artist$Index insert end [lindex $TagList 2]
    
	lappend AllArtists [lindex $TagList 2]
    lappend AllAlbums [lindex $TagList 3]
    lappend AllYears [lindex $TagList 5]
    lappend Local(AllTracks) [lindex $TagList 6]
	lappend AllGenres [lindex $TagList 7]

    # we prepare the next index
	incr Index 
  }

  grid columnconfigure .$Local(TitleFrame) 1 -weight 1
  grid columnconfigure .$Local(TitleFrame) 2 -weight 1

  # we need the count of widgets to destroy all at the next start of titleartist,
  # we use Index here because it was already increased in the last loop 
  set Local(TitleFields) $Index

  # if all items are equal we will fill in the first element - if not we will fill in "" or "Other" in the genre case
  ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).artist [::AuTagger::ElementsAllEqual $AllArtists]
  ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).album [::AuTagger::ElementsAllEqual $AllAlbums]
  ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).year [::AuTagger::ElementsAllEqual $AllYears]
    
  set Config(Genre) [::AuTagger::ElementsAllEqual $AllGenres]
  # correct an empty field for the genre 
  if {$Config(Genre) == ""} { set Config(Genre) "Other" }
 
  ::FR_general::ScrollWindowRefresh $Local(TitleTagsFrame)

  ::FR_general::MakeNormal .$Local(taggerframe).start .$Local(taggerframe).gnudb

  # we configure the scroll increment with help of the first entry-box
  ::FR_general::ScrollWindowIncrement $Local(TitleTagsFrame) $Local(TitleFrame).artist0
}


####################################################################
# we read out the entry-widget and replace the German umlauts
####################################################################
proc ::AuTagger::CheckContent {Widget} {
  variable Config

  # fetch the content of the entry box
  set Content [$Widget get]
  # if we found something...
  if {$Content != ""} {
    # we replace the German umlaut if wished
	if {$Config(ReplaceUmlaut)} {
      regsub -all -- {ü} $Content {ue} Content
      regsub -all -- {Ü} $Content {Ue} Content
      regsub -all -- {ö} $Content {oe} Content
      regsub -all -- {Ö} $Content {Oe} Content
      regsub -all -- {ä} $Content {ae} Content
      regsub -all -- {Ä} $Content {Ae} Content
      regsub -all -- {ß} $Content {ss} Content
    }
  }
  
  return $Content
}


####################################################################
# we read out the content of the entry widget and start the writing
####################################################################
proc ::AuTagger::WriteInfo {} {
  variable Config; variable Local

  # first we catch all information from the entry boxes
  set Album [::AuTagger::CheckContent .$Local(CommonTagsFrame).album]
  set Year [::AuTagger::CheckContent .$Local(CommonTagsFrame).year]
  set Artist [::AuTagger::CheckContent .$Local(CommonTagsFrame).artist]

  # is the general artist entry empty we get the artist information from each title
  # we use this in the next foreach loop
  if {$Artist == ""} {
	# we set a flag
    set NoCommonArtist 1
  } else {
    set NoCommonArtist 0
  }

  # we write all fields to the files
  set Index 0
 
  foreach File $Local(AllFiles) Track $Local(AllTracks) {
    # we fetch the title 
    set Title [::AuTagger::CheckContent .$Local(TitleFrame).title$Index]
    # if no common artist is filled in ...
    if {$NoCommonArtist} {
      # ...we fetch the value also from the title frame 
	  set Artist [::AuTagger::CheckContent .$Local(TitleFrame).artist$Index]
    }
    
    # check if we have a valid track number
	if { [string is integer $Track] == 0 } {
	  # Track isn't an integer we use the index instead
	  set Track $Index
	}
	
	# Title Artist Album Comment Year Track Genre
	::AuFile::AddTags [file join [lindex $Config(OutputDir) 0] $File] \
	[list $Title $Artist $Album $::AuFile::Comment $Year $Track $Config(Genre)]  
    
	# we prepare the next index
	incr Index
  }
}


####################################################################
# Get the information from the internet
####################################################################
proc ::AuTagger::ReadGnuDb {} {
  variable Local; variable Config
  
  # first we try to open the file with the disc id
  if { [file exists $Config(DiscIdFile)] == 0 } {
    ::FR_msgWindow::Add -format Error \
	-msg [msgcat::mc "We can't find the file with the disc id. So we can't fetch the information from the internet."]
    return 1
  }
    
  if { [catch {open $Config(DiscIdFile) r} Fid] } {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "We can't open the file with the disc id."]
    ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "The reason: %s" $Fid]
	return 1
  }
  
  # now we can read out the file...
  set GnuDbId [read $Fid]
  #...and close it again 
  close $Fid

  # The gnu database id should contain: DiscId TrackCnt FrameOffsets TotalSeconds
  set GnuDbIdLength [llength $GnuDbId]
  # GnuDbId has at least four elements at a CD with one track - we check this: 
  if { $GnuDbIdLength < 4 } {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "We read the file with the disc id but the content isn't valid."]
    ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "The disc id hasn't enough elements."]
	return 1
  }

  # if we have (at least) four elements in the GnuDB id string, we can check if the count of offset elements fits
  set TrackCnt [lindex $GnuDbId 1]

  # For each track must exist a FrameOffset plus DiscId, TrackCnt and TotalSeconds (3)
  if { $GnuDbIdLength != [expr {$TrackCnt + 3}] } {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "We read the file with the disc id but the content isn't valid."]
    ::FR_msgWindow::Add -format BodyRed -msg [msgcat::mc "The disc id hasn't enough elements."]
	return 1
  }

  # the GnuDbId seems to be okay we start the request 
  set Local(GnuDbTransferFinished) "-1"

  ::AuTagger::GnuDbSendQueryCmd $GnuDbId 
  vwait ::AuTagger::Local(GnuDbTransferFinished)
      
  # if GnuDbTransferFinished
  # 0...all was successful
  # 1...abort by the user or nothing found
  # 2...hard error
  if { $Local(GnuDbTransferFinished) == 0 } {
    ::AuTagger::GnuDbFillInResults
   	::AuTagger::GnuDbFillInCommonFields
  }
}


####################################################################
# We start here with the GnuDB internet request
#################################################################### 
proc ::AuTagger::GnuDbSendQueryCmd {GnuDbId} {
  global env
  variable Config; variable Local 
 
  # if a proxy used we set the configuration
  if {$Config(Proxy)} {
    http::config -proxyhost $Config(ProxyHost) -proxyport $Config(ProxyPort)
  }

  # we built the query url (including the handshake)
  set Url $Config(GnuDbAddress)
  append Url ? [http::formatQuery cmd "cddb query $GnuDbId" hello "$env(USER) $Config(Domain) $Local(ProgName) $Local(ProgVersion)" proto "5"] 
  
  # we fetch the handshake url
  set CatchReturn [catch {http::geturl $Url -command "::AuTagger::GnuDbParseQuery"} CatchContent]
 
  if {$CatchReturn} {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "An error occurred during connecting to the GnuDB-database: %s" $CatchContent]
    set Local(GnuDbTransferFinished) 2
  }
}


#################################################################### 
# After a query we parse the answer from GnuDB
#################################################################### 
proc ::AuTagger::GnuDbParseQuery {Token} {
  variable Local  
 
  # we fetch the result of the request
  set Data [http::data $Token]
  # now we don't need the token anymore 
  http::cleanup $Token
  
  # we check the return values (error-code) from the GnuDB-database (query)
  if {[regexp {^(200)} $Data]} {
    ::AuTagger::GnuDbSendReadCmd $Data "exact"
  } elseif {[regexp {^(210)} $Data]} {
    ::AuTagger::GnuDbSendReadCmd $Data "multiple"
  } elseif {[regexp {^(211)} $Data]} {
    ::AuTagger::GnuDbSendReadCmd $Data "inexact"
  } elseif {[regexp {^(202)} $Data]} {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Sorry, this CD isn't (yet) in the GnuDB-database"] 
    set Local(GnuDbTransferFinished) 1
  } else {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't get the information from the GnuDB-database. The reason is: %s" $Data]
    set Local(GnuDbTransferFinished) 2
  } 
}


#################################################################### 
# This procedure offers a selection window if there are more than 
# one match and starts the reading   
#################################################################### 
proc ::AuTagger::GnuDbSendReadCmd {Data Matches} {  
  global env
  variable Config; variable Local
  
  # we delete the return code from GnuDB e.g. 200 Found exact match (until terminating `.')
  if {[regexp {(^[0-9]+)(.*)\(until terminating `.'\)\n} $Data]} {
    regsub {(^[0-9]+)(.*)\(until terminating `.'\)\n} $Data "" Data
  } elseif {[regexp {(^[0-9]+)} $Data]} {
    # we delete the return-code from GnuDB if it is only a number
    regsub {(^[0-9]+)} $Data "" Data
  }
 
  # we find the lines with genre-discid-artist/title
  set DataList [regexp -all -inline -- {([a-z]+)([\s]+)([^\s]+)([\s]+)([^\n]*)} $Data]
  set DataListLength [llength $DataList] 
  
  set Local(GnuDbButtonActionFinished) 0
  # GnuDbMultipleValues is the index of the chosen match
  set Local(GnuDbMultipleValues) ""
  
  if {$Matches == "exact"} {
    # 0 represents the first entry, by a exact match - there only exists one...
    set Local(GnuDbMultipleValues) 0
  } else {
    # if we have more than one match, we open a window with the choice possibility
   
    # we generate an empty window with ok/chancel-buttons
    set Window [::FR_emptyBox::Create -window .gdb_matchchoice -title [msgcat::mc "Multiple matches"]]

    # the frames
    frame .$Window.textframe -relief flat
    frame .$Window.buttonframe -relief flat
 
    set TextFrame "$Window.textframe"
    set ButtonFrame "$Window.buttonframe"
 
    pack .$TextFrame -side top -expand yes -fill both -padx 5 -pady 5
    pack .$ButtonFrame -side top -fill x -padx 5 -pady 5
 
    # the text widget  
    set TextWidget [::FR_general::TextwidgetWithScrollbars .$TextFrame]
    
    if {$Matches == "multiple"} {
      .$TextWidget insert end [msgcat::mc "There are multiple entries in the GnuDB-database. Please choose one of them:"]
    } elseif {$Matches == "inexact"} {
      .$TextWidget insert end [msgcat::mc "I only found inexact entries in the GnuDB-database. Please choose one of them or press chancel for nothing:"]
    }
   
    .$TextWidget insert end "\n\n"
    
    # we generate the checkbuttons for the choice
    set CheckBoxList ""

    for {set i 1} {$i < [llength $DataList]} {incr i 6} {
      lappend CheckBoxList [msgcat::mc "%s (categorie: %s)" [lindex $DataList [expr {$i + 4}]] [lindex $DataList $i]]
    }
   
    set CheckBoxListLastIndex [expr {[llength $CheckBoxList] - 1}]
   
    # Index is index and on-value for the checkbutton
    set Index 0
   
    # we set the checkbutton-variable at the first match
    set Local(GnuDbMultipleValues) 0
   
    # we generate for each GnuDB-match one checkbutton
    foreach Arg $CheckBoxList {
      checkbutton .$TextWidget.cb$Index -text $Arg -background white -activebackground white -highlightbackground white \
	  -variable ::AuTagger::Local(GnuDbMultipleValues) -onvalue $Index
      .$TextWidget window create end -window .$TextWidget.cb$Index -padx 10
 
      if {$Index != $CheckBoxListLastIndex} {
        .$TextWidget insert end "\n"
      }
      
      incr Index
    }
 
    # the bottom buttons
    set ButtonText [::FR_general::ButtonsWithSameWidth [list [msgcat::mc "Ok"] [msgcat::mc "Cancel"]]]
 
    button .$ButtonFrame.okay -text [lindex $ButtonText 0] -state normal -width [lindex $ButtonText 2] \
	-command [list ::AuTagger::GnuDbButtonAction $Window 1]
    button .$ButtonFrame.cancel -text [lindex $ButtonText 1] -state normal -width [lindex $ButtonText 2] \
	-command [list ::AuTagger::GnuDbButtonAction $Window 0]
        
    pack .$ButtonFrame.okay -side left -padx 5 -pady 5 -expand yes -fill x
    pack .$ButtonFrame.cancel -side left -padx 5 -pady 5 -expand yes -fill x
  
    bind .$Window <KeyPress-Return> [list .$ButtonFrame.okay invoke]
    bind .$Window <KeyPress-Escape> [list .$ButtonFrame.cancel invoke]
  
    ::FR_emptyBox::Show -window .$Window -returnvalue 0

    vwait ::AuTagger::Local(GnuDbButtonActionFinished)

    # if the Chancel button was pressed we return
    if {$Local(GnuDbMultipleValues) == ""} {
      set Local(GnuDbTransferFinished) 1
      return
    }
  }
      
  set Genre  [lindex $DataList [expr {$Local(GnuDbMultipleValues) * 6 + 1}]]
  set DiscId [lindex $DataList [expr {$Local(GnuDbMultipleValues) * 6 + 3}]]
 
  set Url $Config(GnuDbAddress)
  append Url ? [http::formatQuery cmd "cddb read $Genre $DiscId" hello "$env(USER) $Config(Domain) $Local(ProgName) $Local(ProgVersion)" proto "5"]
    
  set CatchReturn [catch {http::geturl $Url -command "::AuTagger::GnuDbParseRead"} CatchContent]
 
  if {$CatchReturn} {
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "An error occurred during connecting to the GnuDB-database: %s" $CatchContent] 
    set Local(GnuDbTransferFinished) 2
  }
}


#################################################################### 
# The callback function of the choice-window-buttons
#################################################################### 
proc ::AuTagger::GnuDbButtonAction {Window Action} {
  variable Local
  
  # in case of a cancel command we don't set an index for the 
  if {$Action == 0} {
    set Local(GnuDbMultipleValues) "" 
  }
     
  set Local(GnuDbButtonActionFinished) 1 
  focus .
  destroy .$Window
}


#################################################################### 
# This procedure reads the titles from the GnuDB-database and puts 
# they in a cd-text file
#################################################################### 
proc ::AuTagger::GnuDbParseRead {Token} {
  variable Local;
  
  # we fetch the read data
  set Data [http::data $Token] 
  # we don't need the token anymore
  http::cleanup $Token

  #puts "Data: $Data"
  
  # 210 is the code for a successful reading, if not: we return
  if {![regexp {^210 ([^\n]*)} $Data]} {
    set ErrorMsg ""
    regexp {(^[0-9]+)([^\n]+)(\n)} $Data ErrorMsg
    
    ::FR_msgWindow::Add -format Error -msg [msgcat::mc "Couldn't get the information from the GnuDB-database. The reason is: %s" $ErrorMsg]
   
    set Local(GnuDbTransferFinished) 2
    return
  }
  
  # now we find titles, genre, year, albumtitle and artist
  set Album ""; set AlbumArtist ""; set Year ""; set Genre ""
  
  set TitleList [regexp -all -inline -- {TITLE([0-9]+)=([^\n]*)} $Data]
  set TitleListLength [llength $TitleList]
  
  regexp {DGENRE=([^\n]*)} $Data All Genre
  regexp {DYEAR=([^\n]*)} $Data All Year
  regexp {DTITLE=([^\n]*) / ([^\n]*)} $Data All AlbumArtist Album
  
  # we check if genre is a known entry 
  if {[lsearch -exact $::AuFile::GenreCat $Genre] == "-1"} {
    set Genre "Other"
  }
   
  set Local(GnuDbReadResult) [list $Genre $Year $Album $AlbumArtist]
  
  # if there more than one artist on the CD we must read the different artists from the title-entries
  for {set i 1} {$i < $TitleListLength} {incr i 3} {
    set Title ""; set Artist ""
    
    set TitleNumber [lindex $TitleList $i]
    
    set TitleLine [lindex $TitleList [expr {$i + 1}]]
    
    # now we check if the next line the continuation of the last one
    set RunWhileLoop 1
    
    while {$RunWhileLoop} {
      set NextTitleNumberIndex [expr {$i + 3}]
      
      # we don't want to read over the end of the array 
      if {$NextTitleNumberIndex < $TitleListLength} {
        set NextTitleNumber [lindex $TitleList $NextTitleNumberIndex]
      
        # the next line has the same title number -> the next line continues the predecessor line
        if {$TitleNumber == $NextTitleNumber} {
          append TitleLine [lindex $TitleList [expr {$i + 1}]]
          # we look also for further continuations
          incr i 3
        } else {
          # the next line is the next title
          set RunWhileLoop 0
        }
      } else {
		# we reached the end of the array
        set RunWhileLoop 0
      }
    } 
    
	# the title numbers start at zero, we correct this
	incr TitleNumber 

    if {[string first " / " $TitleLine] != -1} {
      regexp {([^\n]*) / ([^\n]*)} $TitleLine All Artist Title
      lappend Local(GnuDbReadResult) $TitleNumber $Artist $Title
    } else {
      lappend Local(GnuDbReadResult) $TitleNumber $AlbumArtist $TitleLine
    }  
  } 

  set Local(GnuDbTransferFinished) 0
}


#################################################################### 
# We fill the GnuDB results into the entry boxes
#################################################################### 
proc ::AuTagger::GnuDbFillInResults {} {
  variable Local
  
  # we save Genre Year Album AlbumArtist for proc GnuDbFillInCommonFields
  eval lappend Local(GnuDbCommonFields) [lrange $Local(GnuDbReadResult) 0 3]
    
  # we don't need Genre Year Album AlbumArtist anymore
  set Local(GnuDbReadResult) [lrange $Local(GnuDbReadResult) 4 end]
  
  set Index 0
  
  foreach "TitleNumber Artist Title" $Local(GnuDbReadResult) {
    # we check if we have a file which contains this track number
	if { [lsearch $Local(AllTracks) $TitleNumber] != -1  } {
      ::AuTagger::InsertEntryContent .$Local(TitleFrame).artist$Index $Artist
      ::AuTagger::InsertEntryContent .$Local(TitleFrame).title$Index $Title
	  incr Index
    }
  }
  
  set Local(GnuDbReadResult) ""
}


#################################################################### 
# Proc fills in the common entry fields
#################################################################### 
proc ::AuTagger::GnuDbFillInCommonFields {} {
  variable Local; variable Config
  
  set GenreList ""; set YearList ""; set AlbumList ""; set AlbumArtistList ""
  
  foreach "Genre Year Album AlbumArtist" $Local(GnuDbCommonFields) {
    lappend GenreList $Genre
    lappend YearList $Year
    lappend AlbumList $Album
    lappend AlbumArtistList $AlbumArtist
  }
  
  set Local(GnuDbCommonFields) ""
  
  set GenreList [::AuTagger::ElementsAllEqual $GenreList]
  set YearList [::AuTagger::ElementsAllEqual $YearList]
  set AlbumList [::AuTagger::ElementsAllEqual $AlbumList]
  set AlbumArtistList [::AuTagger::ElementsAllEqual $AlbumArtistList]

  if {$GenreList != ""} {
    set Config(Genre) $GenreList
  } else {
    set Config(Genre) "Other" 
  }
  
  if {$YearList != ""} {
    ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).year $YearList
  } else {
    ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).year
  }
  
  if {$AlbumList != ""} {
    ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).album $AlbumList
  } else {
    ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).album
  }
  
  if {($AlbumArtistList != "") && ($AlbumArtistList != "Various")} {
    ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).artist $AlbumArtistList
  } else {
    ::AuTagger::InsertEntryContent .$Local(CommonTagsFrame).artist
  }    
}


#################################################################### 
# If wished we delete the GnuDB file
#################################################################### 
proc ::AuTagger::DeleteGnuDbFile {} {
  variable Config 

  if {[file exists $Config(DiscIdFile)]} {
      catch {file delete $Config(DiscIdFile)}
  } 
}

